#include "Gamefield.h"
#include <fstream>
#include <iostream>
#include <cassert>
#include <cmath>

Gamefield::Gamefield()
{
    //ctor
}

Gamefield::~Gamefield()
{
    //dtor
}

const bool Gamefield::Init(const std::string& filename, Player& player)
{
    std::ifstream file(filename.c_str());
    if(file.fail())
    {
        std::cout<<"Could not open " << filename << "!" << std::endl;
        return false;
    }
    int curRowIndex = 0;
    while(true)
    {
        std::string curLine;
        std::getline(file, curLine);
        if(curLine == "")
        {
            //empty file?
            if(Cells.size() == 0)
            {
                std::cout<<filename<<" is empty!"<<std::endl;
                return false;
            }
            return true;
        }
        int curCellIndex = 0;
        for(std::string::iterator it = curLine.begin(); it != curLine.end(); ++it)
        {
            if(*it == PLAYER_CHAR)
            {
                *it = EMPTY_CHAR;
                player.X = curCellIndex;
                player.Y = curRowIndex;
            }
            ++curCellIndex;
        }
        Row curRow;
        curRow.resize(curLine.length());
        std::copy(curLine.begin(), curLine.end(), curRow.begin());
        ++curRowIndex;
        Cells.push_back(curRow);
    }
    //won't reach this.
}

void Gamefield::Display(const Player& player)
{
    std::string output = "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n";
    for(int Y = 0; Y < Cells.size(); ++Y)
    {
        for(int X = 0; X < Cells[Y].size(); ++X)
        {
            //player here?
            if(X == player.X && Y == player.Y)
            {
                output += 'P';
            }
            else
            {
                output += Cells[Y][X];
            }
        }
        output += '\n';
    }
    std::cout<<output<<std::endl;
}

void Gamefield::TryMove(Player& player, int deltaX, int deltaY)
{
    assert(deltaX == 0 or deltaY == 0);
    assert(abs(deltaX) < 2); //-1, 0, 1
    assert(abs(deltaY) < 2); //-1, 0, 1
    //simply fits?
    int posX = player.X + deltaX;
    int posY = player.Y + deltaY;
    //out of bounds?
    if(posY < 0 || posY >= Cells.size() || posX < 0 || posX >= Cells[posY].size())
        return;
    //empty?
    if(Cells[posY][posX] == EMPTY_CHAR || Cells[posY][posX] == EXIT_CHAR)
    {
        player.X = posX;
        player.Y = posY;
        return;
    }
    //no crate?
    if(Cells[posY][posX] != CRATE_CHAR )
        return;
    int x2 = posX + deltaX;
    int y2 = posY + deltaY;
    //out of bounds?
    if(y2 < 0 || y2 >= Cells.size() || x2 < 0 || x2 >= Cells[y2].size())
        return;
    //not empty?
    //BUG: exit is not empty!
    if(Cells[y2][x2] != EMPTY_CHAR)
        return;
    //move crate & player
    Cells[posY][posX] = EMPTY_CHAR;
    Cells[y2][x2] = CRATE_CHAR;
    player.X = posX;
    player.Y = posY;
}
